@@ -39,13 +39,9 @@
 //!   Note that enabling this feature has some costs, as it means forgoing some optimizations.
 //!   Ambiguities may be introduced when parsing that would not otherwise exist.
 //!
-//!   If you are using this feature, **please leave a comment**
-//!   [on this discussion](https://github.com/time-rs/time/discussions/306) with your use case. If
-//!   there is not sufficient demand for this feature, it will be dropped in a future release.
-//!
 //! - `serde`
 //!
-//!   Enables [serde](https://docs.rs/serde) support for all types.
+//!   Enables [serde](https://docs.rs/serde) support for all types except [`Instant`].
 //!
 //! - `serde-human-readable` (_implicitly enables `serde`, `formatting`, and `parsing`_)
 //!
@@ -53,6 +49,17 @@
 //!   serializer, not the user. If this feature is not enabled or if the serializer requests a
 //!   non-human-readable format, a format optimized for binary representation will be used.
 //!
+//!   Libraries should never enable this feature, as the decision of what format to use should be up
+//!   to the user.
+//!
+//! - `serde-well-known` (_implicitly enables `serde-human-readable`_)
+//!
+//!   _This feature flag is deprecated and will be removed in a future breaking release. Use the
+//!   `serde-human-readable` feature instead._
+//!
+//!   Enables support for serializing and deserializing well-known formats using serde's
+//!   [`#[with]` attribute](https://serde.rs/field-attrs.html#with).
+//!
 //! - `rand`
 //!
 //!   Enables [rand](https://docs.rs/rand) support for all types.
@@ -61,25 +68,30 @@
 //!
 //!   Enables [quickcheck](https://docs.rs/quickcheck) support for all types except [`Instant`].
 //!
-//! One pseudo-feature flag that is only available to end users is the `unsound_local_offset` cfg.
-//! As the name indicates, using the feature is unsound, and [may cause unexpected segmentation
-//! faults](https://github.com/time-rs/time/issues/293). Unlike other flags, this is deliberately
-//! only available to end users; this is to ensure that a user doesn't have unsound behavior without
-//! knowing it. To enable this behavior, you must use `RUSTFLAGS="--cfg unsound_local_offset" cargo
-//! build` or similar. Note: This flag is _not tested anywhere_, including in the regular test of
-//! the powerset of all feature flags. Use at your own risk. Without this flag, any method that
-//! requires the local offset will return the `Err` variant.
+//! - `wasm-bindgen`
+//!
+//!   Enables [wasm-bindgen](https://github.com/rustwasm/wasm-bindgen) support for converting
+//!   [JavaScript dates](https://rustwasm.github.io/wasm-bindgen/api/js_sys/struct.Date.html), as
+//!   well as obtaining the UTC offset from JavaScript.
+//!
+//! <small>
+//! One feature only available to end users is the <code>unsound_local_offset</code> cfg. This
+//! enables obtaining the system's UTC offset even when it is unsound. To enable this, use the
+//! <code>RUSTFLAGS</code> environment variable. This is untested and officially unsupported. Do not
+//! use this unless you understand the risk.
+//! </small>
 
 #![doc(html_playground_url = "https://play.rust-lang.org")]
-#![cfg_attr(__time_03_docs, feature(doc_cfg))]
-#![cfg_attr(
-    __time_03_docs,
-    deny(rustdoc::broken_intra_doc_links, rustdoc::private_intra_doc_links)
-)]
+#![cfg_attr(__time_03_docs, feature(doc_auto_cfg, doc_notable_trait))]
 #![cfg_attr(not(feature = "std"), no_std)]
 #![deny(
     anonymous_parameters,
     clippy::all,
+    clippy::alloc_instead_of_core,
+    clippy::explicit_auto_deref,
+    clippy::obfuscated_if_else,
+    clippy::std_instead_of_core,
+    clippy::undocumented_unsafe_blocks,
     const_err,
     illegal_floating_point_literal_pattern,
     late_bound_lifetime_arguments,
@@ -89,8 +101,10 @@
     trivial_casts,
     trivial_numeric_casts,
     unreachable_pub,
-    unsafe_code,
-    unused_extern_crates
+    unsafe_op_in_unsafe_fn,
+    unused_extern_crates,
+    rustdoc::broken_intra_doc_links,
+    rustdoc::private_intra_doc_links
 )]
 #![warn(
     clippy::dbg_macro,
@@ -98,10 +112,10 @@
     clippy::get_unwrap,
     clippy::missing_docs_in_private_items,
     clippy::nursery,
-    clippy::pedantic,
     clippy::print_stdout,
     clippy::todo,
     clippy::unimplemented,
+    clippy::unnested_or_patterns,
     clippy::unwrap_in_result,
     clippy::unwrap_used,
     clippy::use_debug,
@@ -111,19 +125,7 @@
     unused_qualifications,
     variant_size_differences
 )]
-#![allow(
-    clippy::cast_lossless,
-    clippy::cast_possible_truncation,
-    clippy::cast_possible_wrap,
-    clippy::cast_precision_loss,
-    clippy::cast_sign_loss,
-    clippy::enum_glob_use,
-    clippy::map_err_ignore,
-    clippy::missing_errors_doc,
-    clippy::must_use_candidate,
-    clippy::redundant_pub_crate,
-    unstable_name_collisions
-)]
+#![allow(clippy::redundant_pub_crate)]
 #![doc(html_favicon_url = "https://avatars0.githubusercontent.com/u/55999857")]
 #![doc(html_logo_url = "https://avatars0.githubusercontent.com/u/55999857")]
 #![doc(test(attr(deny(warnings))))]
@@ -133,6 +135,7 @@
 extern crate alloc;
 
 // region: macros
+/// Helper macro for easily implementing `OpAssign`.
 macro_rules! __impl_assign {
     ($sym:tt $op:ident $fn:ident $target:ty : $($(#[$attr:meta])* $t:ty),+) => {$(
         #[allow(unused_qualifications)]
@@ -189,27 +192,13 @@ macro_rules! div_floor {
     }};
 }
 
-/// Euclidean remainder. Useful for `const` contexts.
-macro_rules! rem_euclid {
-    ($a:expr, $b:expr) => {{
-        let _a = $a;
-        let _b = $b;
-        let r = _a % _b;
-        if r < 0 {
-            if _b < 0 { r - _b } else { r + _b }
-        } else {
-            r
-        }
-    }};
-}
-
 /// Cascade an out-of-bounds value.
 macro_rules! cascade {
     (@ordinal ordinal) => {};
     (@year year) => {};
 
     // Cascade an out-of-bounds value from "from" to "to".
-    ($(!$from_not_mut:ident)? $from:ident in $min:literal.. $max:literal => $to:tt) => {
+    ($from:ident in $min:literal.. $max:literal => $to:tt) => {
         #[allow(unused_comparisons, unused_assignments)]
         if $from >= $max {
             $from -= $max - $min;
@@ -226,12 +215,12 @@ macro_rules! cascade {
         cascade!(@ordinal $ordinal);
         cascade!(@year $year);
         #[allow(unused_assignments)]
-        if $ordinal > crate::util::days_in_year($year) {
+        if $ordinal > crate::util::days_in_year($year) as i16 {
+            $ordinal -= crate::util::days_in_year($year) as i16;
             $year += 1;
-            $ordinal = 1;
-        } else if $ordinal == 0 {
+        } else if $ordinal < 1 {
             $year -= 1;
-            $ordinal = crate::util::days_in_year($year);
+            $ordinal += crate::util::days_in_year($year) as i16;
         }
     };
 }
@@ -293,59 +282,50 @@ macro_rules! const_try_opt {
         }
     };
 }
+
+/// Try to unwrap an expression, panicking if not possible.
+///
+/// This is similar to `$e.expect($message)`, but is usable in `const` contexts.
+macro_rules! expect_opt {
+    ($e:expr, $message:literal) => {
+        match $e {
+            Some(value) => value,
+            None => crate::expect_failed($message),
+        }
+    };
+}
 // endregion macros
 
-/// The [`Date`] struct and its associated `impl`s.
 mod date;
-/// The [`Duration`] struct and its associated `impl`s.
 mod duration;
-/// Various error types returned by methods in the time crate.
 pub mod error;
-/// Extension traits.
 pub mod ext;
 #[cfg(any(feature = "formatting", feature = "parsing"))]
-#[cfg_attr(
-    __time_03_docs,
-    doc(cfg(any(feature = "formatting", feature = "parsing")))
-)]
 pub mod format_description;
 #[cfg(feature = "formatting")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "formatting")))]
 pub mod formatting;
-/// The [`Instant`] struct and its associated `impl`s.
 #[cfg(feature = "std")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
 mod instant;
-/// Macros to construct statically known values.
 #[cfg(feature = "macros")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "macros")))]
 pub mod macros;
 mod month;
-/// The [`OffsetDateTime`] struct and its associated `impl`s.
 mod offset_date_time;
 #[cfg(feature = "parsing")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "parsing")))]
 pub mod parsing;
-/// The [`PrimitiveDateTime`] struct and its associated `impl`s.
 mod primitive_date_time;
 #[cfg(feature = "quickcheck")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "quickcheck")))]
 mod quickcheck;
 #[cfg(feature = "rand")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "rand")))]
 mod rand;
 #[cfg(feature = "serde")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "serde")))]
 #[allow(missing_copy_implementations, missing_debug_implementations)]
 pub mod serde;
+mod sys;
 #[cfg(test)]
 mod tests;
-/// The [`Time`] struct and its associated `impl`s.
 mod time;
-/// The [`UtcOffset`] struct and its associated `impl`s.
 mod utc_offset;
 pub mod util;
-/// Days of the week.
 mod weekday;
 
 pub use crate::date::Date;
@@ -362,3 +342,11 @@ pub use crate::weekday::Weekday;
 
 /// An alias for [`std::result::Result`] with a generic error from the time crate.
 pub type Result<T> = core::result::Result<T, Error>;
+
+/// This is a separate function to reduce the code size of `expect_opt!`.
+#[inline(never)]
+#[cold]
+#[track_caller]
+const fn expect_failed(message: &str) -> ! {
+    panic!("{}", message)
+}
