@@ -1,3 +1,5 @@
+//! The [`OffsetDateTime`] struct and its associated `impl`s.
+
 use core::cmp::Ordering;
 #[cfg(feature = "std")]
 use core::convert::From;
@@ -10,10 +12,13 @@ use std::io;
 #[cfg(feature = "std")]
 use std::time::SystemTime;
 
+use crate::date::{MAX_YEAR, MIN_YEAR};
 #[cfg(feature = "formatting")]
 use crate::formatting::Formattable;
 #[cfg(feature = "parsing")]
 use crate::parsing::Parsable;
+#[cfg(feature = "parsing")]
+use crate::util;
 use crate::{error, Date, Duration, Month, PrimitiveDateTime, Time, UtcOffset, Weekday};
 
 /// The Julian day of the Unix epoch.
@@ -22,13 +27,10 @@ const UNIX_EPOCH_JULIAN_DAY: i32 = Date::__from_ordinal_date_unchecked(1970, 1).
 /// A [`PrimitiveDateTime`] with a [`UtcOffset`].
 ///
 /// All comparisons are performed using the UTC time.
-// Internally, an `OffsetDateTime` is a thin wrapper around a [`PrimitiveDateTime`] coupled with a
-// [`UtcOffset`]. This offset is added to the date, time, or datetime as necessary for presentation
-// or returning from a function.
-#[derive(Debug, Clone, Copy, Eq)]
+#[derive(Clone, Copy, Eq)]
 pub struct OffsetDateTime {
-    /// The [`PrimitiveDateTime`], which is _always_ UTC.
-    pub(crate) utc_datetime: PrimitiveDateTime,
+    /// The [`PrimitiveDateTime`], which is _always_ in the stored offset.
+    pub(crate) local_datetime: PrimitiveDateTime,
     /// The [`UtcOffset`], which will be added to the [`PrimitiveDateTime`] as necessary.
     pub(crate) offset: UtcOffset,
 }
@@ -53,8 +55,21 @@ impl OffsetDateTime {
     /// assert_eq!(OffsetDateTime::now_utc().offset(), offset!(UTC));
     /// ```
     #[cfg(feature = "std")]
-    #[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
     pub fn now_utc() -> Self {
+        #[cfg(all(
+            target_arch = "wasm32",
+            not(any(target_os = "emscripten", target_os = "wasi")),
+            feature = "wasm-bindgen"
+        ))]
+        {
+            js_sys::Date::new_0().into()
+        }
+
+        #[cfg(not(all(
+            target_arch = "wasm32",
+            not(any(target_os = "emscripten", target_os = "wasi")),
+            feature = "wasm-bindgen"
+        )))]
         SystemTime::now().into()
     }
 
@@ -68,7 +83,6 @@ impl OffsetDateTime {
     /// # }
     /// ```
     #[cfg(feature = "local-offset")]
-    #[cfg_attr(__time_03_docs, doc(cfg(feature = "local-offset")))]
     pub fn now_local() -> Result<Self, error::IndeterminateOffset> {
         let t = Self::now_utc();
         Ok(t.to_offset(UtcOffset::local_offset_at(t)?))
@@ -86,7 +100,7 @@ impl OffsetDateTime {
     ///     1999,
     /// );
     ///
-    /// // Let's see what time Sydney's new year's celebration is in New York // and Los Angeles.
+    /// // Let's see what time Sydney's new year's celebration is in New York and Los Angeles.
     ///
     /// // Construct midnight on new year's in Sydney.
     /// let sydney = datetime!(2000-01-01 0:00 +11);
@@ -96,11 +110,73 @@ impl OffsetDateTime {
     /// assert_eq!(new_york.hour(), 8);
     /// assert_eq!(los_angeles.hour(), 5);
     /// ```
+    ///
+    /// # Panics
+    ///
+    /// This method panics if the local date-time in the new offset is outside the supported range.
     pub const fn to_offset(self, offset: UtcOffset) -> Self {
-        Self {
-            utc_datetime: self.utc_datetime,
-            offset,
+        if self.offset.whole_hours() == offset.whole_hours()
+            && self.offset.minutes_past_hour() == offset.minutes_past_hour()
+            && self.offset.seconds_past_minute() == offset.seconds_past_minute()
+        {
+            return self;
         }
+
+        let (year, ordinal, time) = self.to_offset_raw(offset);
+
+        if year > MAX_YEAR || year < MIN_YEAR {
+            panic!("local datetime out of valid range");
+        }
+
+        Date::__from_ordinal_date_unchecked(year, ordinal)
+            .with_time(time)
+            .assume_offset(offset)
+    }
+
+    /// Equivalent to `.to_offset(UtcOffset::UTC)`, but returning the year, ordinal, and time. This
+    /// avoids constructing an invalid [`Date`] if the new value is out of range.
+    const fn to_offset_raw(self, offset: UtcOffset) -> (i32, u16, Time) {
+        let from = self.offset;
+        let to = offset;
+
+        // Fast path for when no conversion is necessary.
+        if from.whole_hours() == to.whole_hours()
+            && from.minutes_past_hour() == to.minutes_past_hour()
+            && from.seconds_past_minute() == to.seconds_past_minute()
+        {
+            return (self.year(), self.ordinal(), self.time());
+        }
+
+        let mut second = self.second() as i16 - from.seconds_past_minute() as i16
+            + to.seconds_past_minute() as i16;
+        let mut minute =
+            self.minute() as i16 - from.minutes_past_hour() as i16 + to.minutes_past_hour() as i16;
+        let mut hour = self.hour() as i8 - from.whole_hours() + to.whole_hours();
+        let (mut year, ordinal) = self.to_ordinal_date();
+        let mut ordinal = ordinal as i16;
+
+        // Cascade the values twice. This is needed because the values are adjusted twice above.
+        cascade!(second in 0..60 => minute);
+        cascade!(second in 0..60 => minute);
+        cascade!(minute in 0..60 => hour);
+        cascade!(minute in 0..60 => hour);
+        cascade!(hour in 0..24 => ordinal);
+        cascade!(hour in 0..24 => ordinal);
+        cascade!(ordinal => year);
+
+        debug_assert!(ordinal > 0);
+        debug_assert!(ordinal <= crate::util::days_in_year(year) as i16);
+
+        (
+            year,
+            ordinal as _,
+            Time::__from_hms_nanos_unchecked(
+                hour as _,
+                minute as _,
+                second as _,
+                self.nanosecond(),
+            ),
+        )
     }
 
     // region: constructors
@@ -147,7 +223,7 @@ impl OffsetDateTime {
             UNIX_EPOCH_JULIAN_DAY + div_floor!(timestamp, 86_400) as i32,
         );
 
-        let seconds_within_day = rem_euclid!(timestamp, 86_400);
+        let seconds_within_day = timestamp.rem_euclid(86_400);
         let time = Time::__from_hms_nanos_unchecked(
             (seconds_within_day / 3_600) as _,
             ((seconds_within_day % 3_600) / 60) as _,
@@ -178,12 +254,12 @@ impl OffsetDateTime {
         ));
 
         Ok(datetime
-            .utc_datetime
+            .local_datetime
             .replace_time(Time::__from_hms_nanos_unchecked(
-                datetime.utc_datetime.hour(),
-                datetime.utc_datetime.minute(),
-                datetime.utc_datetime.second(),
-                rem_euclid!(timestamp, 1_000_000_000) as u32,
+                datetime.local_datetime.hour(),
+                datetime.local_datetime.minute(),
+                datetime.local_datetime.second(),
+                timestamp.rem_euclid(1_000_000_000) as u32,
             ))
             .assume_utc())
     }
@@ -209,12 +285,14 @@ impl OffsetDateTime {
     /// assert_eq!(datetime!(1970-01-01 0:00 -1).unix_timestamp(), 3_600);
     /// ```
     pub const fn unix_timestamp(self) -> i64 {
+        let offset = self.offset.whole_seconds() as i64;
+
         let days =
-            (self.utc_datetime.to_julian_day() as i64 - UNIX_EPOCH_JULIAN_DAY as i64) * 86_400;
-        let hours = self.utc_datetime.hour() as i64 * 3_600;
-        let minutes = self.utc_datetime.minute() as i64 * 60;
-        let seconds = self.utc_datetime.second() as i64;
-        days + hours + minutes + seconds
+            (self.local_datetime.to_julian_day() as i64 - UNIX_EPOCH_JULIAN_DAY as i64) * 86_400;
+        let hours = self.local_datetime.hour() as i64 * 3_600;
+        let minutes = self.local_datetime.minute() as i64 * 60;
+        let seconds = self.local_datetime.second() as i64;
+        days + hours + minutes + seconds - offset
     }
 
     /// Get the Unix timestamp in nanoseconds.
@@ -228,7 +306,7 @@ impl OffsetDateTime {
     /// );
     /// ```
     pub const fn unix_timestamp_nanos(self) -> i128 {
-        self.unix_timestamp() as i128 * 1_000_000_000 + self.utc_datetime.nanosecond() as i128
+        self.unix_timestamp() as i128 * 1_000_000_000 + self.nanosecond() as i128
     }
 
     /// Get the [`Date`] in the stored offset.
@@ -244,17 +322,7 @@ impl OffsetDateTime {
     /// );
     /// ```
     pub const fn date(self) -> Date {
-        let mut second = self.utc_datetime.second() as i8 + self.offset.seconds_past_minute();
-        let mut minute = self.utc_datetime.minute() as i8 + self.offset.minutes_past_hour();
-        let mut hour = self.utc_datetime.hour() as i8 + self.offset.whole_hours();
-        let (mut year, mut ordinal) = self.utc_datetime.date.to_ordinal_date();
-
-        cascade!(second in 0..60 => minute);
-        cascade!(minute in 0..60 => hour);
-        cascade!(hour in 0..24 => ordinal);
-        cascade!(ordinal => year);
-
-        Date::__from_ordinal_date_unchecked(year, ordinal)
+        self.local_datetime.date()
     }
 
     /// Get the [`Time`] in the stored offset.
@@ -270,19 +338,7 @@ impl OffsetDateTime {
     /// );
     /// ```
     pub const fn time(self) -> Time {
-        let mut second = self.utc_datetime.second() as i8 + self.offset.seconds_past_minute();
-        let mut minute = self.utc_datetime.minute() as i8 + self.offset.minutes_past_hour();
-        let mut hour = self.utc_datetime.hour() as i8 + self.offset.whole_hours();
-
-        cascade!(second in 0..60 => minute);
-        cascade!(minute in 0..60 => hour);
-
-        Time::__from_hms_nanos_unchecked(
-            rem_euclid!(hour, 24) as _,
-            minute as _,
-            second as _,
-            self.utc_datetime.nanosecond(),
-        )
+        self.local_datetime.time()
     }
 
     // region: date getters
@@ -300,17 +356,7 @@ impl OffsetDateTime {
     /// assert_eq!(datetime!(2020-01-01 0:00 UTC).year(), 2020);
     /// ```
     pub const fn year(self) -> i32 {
-        let mut second = self.utc_datetime.second() as i8 + self.offset.seconds_past_minute();
-        let mut minute = self.utc_datetime.minute() as i8 + self.offset.minutes_past_hour();
-        let mut hour = self.utc_datetime.hour() as i8 + self.offset.whole_hours();
-        let (mut year, mut ordinal) = self.utc_datetime.date.to_ordinal_date();
-
-        cascade!(second in 0..60 => minute);
-        cascade!(minute in 0..60 => hour);
-        cascade!(hour in 0..24 => ordinal);
-        cascade!(ordinal => year);
-
-        year
+        self.date().year()
     }
 
     /// Get the month of the date in the stored offset.
@@ -363,17 +409,7 @@ impl OffsetDateTime {
     /// );
     /// ```
     pub const fn ordinal(self) -> u16 {
-        let mut second = self.utc_datetime.second() as i8 + self.offset.seconds_past_minute();
-        let mut minute = self.utc_datetime.minute() as i8 + self.offset.minutes_past_hour();
-        let mut hour = self.utc_datetime.hour() as i8 + self.offset.whole_hours();
-        let (mut year, mut ordinal) = self.utc_datetime.date.to_ordinal_date();
-
-        cascade!(second in 0..60 => minute);
-        cascade!(minute in 0..60 => hour);
-        cascade!(hour in 0..24 => ordinal);
-        cascade!(ordinal => year);
-
-        ordinal
+        self.date().ordinal()
     }
 
     /// Get the ISO week number of the date in the stored offset.
@@ -583,13 +619,7 @@ impl OffsetDateTime {
     /// );
     /// ```
     pub const fn hour(self) -> u8 {
-        let mut second = self.utc_datetime.second() as i8 + self.offset.seconds_past_minute();
-        let mut minute = self.utc_datetime.minute() as i8 + self.offset.minutes_past_hour();
-        let mut hour = self.utc_datetime.hour() as i8 + self.offset.whole_hours();
-
-        cascade!(second in 0..60 => minute);
-        cascade!(minute in 0..60 => hour);
-        rem_euclid!(hour, 24) as _
+        self.time().hour()
     }
 
     /// Get the minute within the hour in the stored offset.
@@ -607,11 +637,7 @@ impl OffsetDateTime {
     /// );
     /// ```
     pub const fn minute(self) -> u8 {
-        let mut second = self.utc_datetime.second() as i8 + self.offset.seconds_past_minute();
-        let mut minute = self.utc_datetime.minute() as i8 + self.offset.minutes_past_hour();
-
-        cascade!(second in 0..60 => minute);
-        rem_euclid!(minute, 60) as _
+        self.time().minute()
     }
 
     /// Get the second within the minute in the stored offset.
@@ -629,8 +655,7 @@ impl OffsetDateTime {
     /// );
     /// ```
     pub const fn second(self) -> u8 {
-        let second = self.utc_datetime.second() as i8 + self.offset.seconds_past_minute();
-        rem_euclid!(second, 60) as _
+        self.time().second()
     }
 
     // Because a `UtcOffset` is limited in resolution to one second, any subsecond value will not
@@ -646,7 +671,7 @@ impl OffsetDateTime {
     /// assert_eq!(datetime!(2019-01-01 23:59:59.999 UTC).millisecond(), 999);
     /// ```
     pub const fn millisecond(self) -> u16 {
-        self.utc_datetime.millisecond()
+        self.time().millisecond()
     }
 
     /// Get the microseconds within the second in the stored offset.
@@ -662,7 +687,7 @@ impl OffsetDateTime {
     /// );
     /// ```
     pub const fn microsecond(self) -> u32 {
-        self.utc_datetime.microsecond()
+        self.time().microsecond()
     }
 
     /// Get the nanoseconds within the second in the stored offset.
@@ -678,10 +703,116 @@ impl OffsetDateTime {
     /// );
     /// ```
     pub const fn nanosecond(self) -> u32 {
-        self.utc_datetime.nanosecond()
+        self.time().nanosecond()
     }
     // endregion time getters
     // endregion getters
+
+    // region: checked arithmetic
+    /// Computes `self + duration`, returning `None` if an overflow occurred.
+    ///
+    /// ```
+    /// # use time::{Date, ext::NumericalDuration};
+    /// # use time::macros::{datetime, offset};
+    /// let datetime = Date::MIN.midnight().assume_offset(offset!(+10));
+    /// assert_eq!(datetime.checked_add((-2).days()), None);
+    ///
+    /// let datetime = Date::MAX.midnight().assume_offset(offset!(+10));
+    /// assert_eq!(datetime.checked_add(2.days()), None);
+    ///
+    /// assert_eq!(
+    ///     datetime!(2019 - 11 - 25 15:30 +10).checked_add(27.hours()),
+    ///     Some(datetime!(2019 - 11 - 26 18:30 +10))
+    /// );
+    /// ```
+    pub const fn checked_add(self, duration: Duration) -> Option<Self> {
+        Some(const_try_opt!(self.local_datetime.checked_add(duration)).assume_offset(self.offset))
+    }
+
+    /// Computes `self - duration`, returning `None` if an overflow occurred.
+    ///
+    /// ```
+    /// # use time::{Date, ext::NumericalDuration};
+    /// # use time::macros::{datetime, offset};
+    /// let datetime = Date::MIN.midnight().assume_offset(offset!(+10));
+    /// assert_eq!(datetime.checked_sub(2.days()), None);
+    ///
+    /// let datetime = Date::MAX.midnight().assume_offset(offset!(+10));
+    /// assert_eq!(datetime.checked_sub((-2).days()), None);
+    ///
+    /// assert_eq!(
+    ///     datetime!(2019 - 11 - 25 15:30 +10).checked_sub(27.hours()),
+    ///     Some(datetime!(2019 - 11 - 24 12:30 +10))
+    /// );
+    /// ```
+    pub const fn checked_sub(self, duration: Duration) -> Option<Self> {
+        Some(const_try_opt!(self.local_datetime.checked_sub(duration)).assume_offset(self.offset))
+    }
+    // endregion: checked arithmetic
+
+    // region: saturating arithmetic
+    /// Computes `self + duration`, saturating value on overflow.
+    ///
+    /// ```
+    /// # use time::ext::NumericalDuration;
+    /// # use time::macros::datetime;
+    /// assert_eq!(
+    ///     datetime!(-999999 - 01 - 01 0:00 +10).saturating_add((-2).days()),
+    ///     datetime!(-999999 - 01 - 01 0:00 +10)
+    /// );
+    ///
+    /// assert_eq!(
+    ///     datetime!(+999999 - 12 - 31 23:59:59.999_999_999 +10).saturating_add(2.days()),
+    ///     datetime!(+999999 - 12 - 31 23:59:59.999_999_999 +10)
+    /// );
+    ///
+    /// assert_eq!(
+    ///     datetime!(2019 - 11 - 25 15:30 +10).saturating_add(27.hours()),
+    ///     datetime!(2019 - 11 - 26 18:30 +10)
+    /// );
+    /// ```
+    pub const fn saturating_add(self, duration: Duration) -> Self {
+        if let Some(datetime) = self.checked_add(duration) {
+            datetime
+        } else if duration.is_negative() {
+            PrimitiveDateTime::MIN.assume_offset(self.offset)
+        } else {
+            debug_assert!(duration.is_positive());
+            PrimitiveDateTime::MAX.assume_offset(self.offset)
+        }
+    }
+
+    /// Computes `self - duration`, saturating value on overflow.
+    ///
+    /// ```
+    /// # use time::ext::NumericalDuration;
+    /// # use time::macros::datetime;
+    /// assert_eq!(
+    ///     datetime!(-999999 - 01 - 01 0:00 +10).saturating_sub(2.days()),
+    ///     datetime!(-999999 - 01 - 01 0:00 +10)
+    /// );
+    ///
+    /// assert_eq!(
+    ///     datetime!(+999999 - 12 - 31 23:59:59.999_999_999 +10).saturating_sub((-2).days()),
+    ///     datetime!(+999999 - 12 - 31 23:59:59.999_999_999 +10)
+    /// );
+    ///
+    /// assert_eq!(
+    ///     datetime!(2019 - 11 - 25 15:30 +10).saturating_sub(27.hours()),
+    ///     datetime!(2019 - 11 - 24 12:30 +10)
+    /// );
+    /// ```
+    pub const fn saturating_sub(self, duration: Duration) -> Self {
+        if let Some(datetime) = self.checked_sub(duration) {
+            datetime
+        } else if duration.is_negative() {
+            PrimitiveDateTime::MAX.assume_offset(self.offset)
+        } else {
+            debug_assert!(duration.is_positive());
+            PrimitiveDateTime::MIN.assume_offset(self.offset)
+        }
+    }
+    // endregion: saturating arithmetic
 }
 
 // region: replacement
@@ -707,8 +838,7 @@ impl OffsetDateTime {
     /// ```
     #[must_use = "This method does not mutate the original `OffsetDateTime`."]
     pub const fn replace_time(self, time: Time) -> Self {
-        self.utc_datetime
-            .utc_to_offset(self.offset)
+        self.local_datetime
             .replace_time(time)
             .assume_offset(self.offset)
     }
@@ -729,8 +859,7 @@ impl OffsetDateTime {
     /// ```
     #[must_use = "This method does not mutate the original `OffsetDateTime`."]
     pub const fn replace_date(self, date: Date) -> Self {
-        self.utc_datetime
-            .utc_to_offset(self.offset)
+        self.local_datetime
             .replace_date(date)
             .assume_offset(self.offset)
     }
@@ -765,27 +894,169 @@ impl OffsetDateTime {
     /// ```
     #[must_use = "This method does not mutate the original `OffsetDateTime`."]
     pub const fn replace_offset(self, offset: UtcOffset) -> Self {
-        self.utc_datetime.assume_offset(offset)
+        self.local_datetime.assume_offset(offset)
+    }
+
+    /// Replace the year. The month and day will be unchanged.
+    ///
+    /// ```rust
+    /// # use time::macros::datetime;
+    /// assert_eq!(
+    ///     datetime!(2022 - 02 - 18 12:00 +01).replace_year(2019),
+    ///     Ok(datetime!(2019 - 02 - 18 12:00 +01))
+    /// );
+    /// assert!(datetime!(2022 - 02 - 18 12:00 +01).replace_year(-1_000_000_000).is_err()); // -1_000_000_000 isn't a valid year
+    /// assert!(datetime!(2022 - 02 - 18 12:00 +01).replace_year(1_000_000_000).is_err()); // 1_000_000_000 isn't a valid year
+    /// ```
+    pub const fn replace_year(self, year: i32) -> Result<Self, error::ComponentRange> {
+        Ok(const_try!(self.local_datetime.replace_year(year)).assume_offset(self.offset))
+    }
+
+    /// Replace the month of the year.
+    ///
+    /// ```rust
+    /// # use time::macros::datetime;
+    /// # use time::Month;
+    /// assert_eq!(
+    ///     datetime!(2022 - 02 - 18 12:00 +01).replace_month(Month::January),
+    ///     Ok(datetime!(2022 - 01 - 18 12:00 +01))
+    /// );
+    /// assert!(datetime!(2022 - 01 - 30 12:00 +01).replace_month(Month::February).is_err()); // 30 isn't a valid day in February
+    /// ```
+    pub const fn replace_month(self, month: Month) -> Result<Self, error::ComponentRange> {
+        Ok(const_try!(self.local_datetime.replace_month(month)).assume_offset(self.offset))
+    }
+
+    /// Replace the day of the month.
+    ///
+    /// ```rust
+    /// # use time::macros::datetime;
+    /// assert_eq!(
+    ///     datetime!(2022 - 02 - 18 12:00 +01).replace_day(1),
+    ///     Ok(datetime!(2022 - 02 - 01 12:00 +01))
+    /// );
+    /// assert!(datetime!(2022 - 02 - 18 12:00 +01).replace_day(0).is_err()); // 00 isn't a valid day
+    /// assert!(datetime!(2022 - 02 - 18 12:00 +01).replace_day(30).is_err()); // 30 isn't a valid day in February
+    /// ```
+    pub const fn replace_day(self, day: u8) -> Result<Self, error::ComponentRange> {
+        Ok(const_try!(self.local_datetime.replace_day(day)).assume_offset(self.offset))
+    }
+
+    /// Replace the clock hour.
+    ///
+    /// ```rust
+    /// # use time::macros::datetime;
+    /// assert_eq!(
+    ///     datetime!(2022 - 02 - 18 01:02:03.004_005_006 +01).replace_hour(7),
+    ///     Ok(datetime!(2022 - 02 - 18 07:02:03.004_005_006 +01))
+    /// );
+    /// assert!(datetime!(2022 - 02 - 18 01:02:03.004_005_006 +01).replace_hour(24).is_err()); // 24 isn't a valid hour
+    /// ```
+    pub const fn replace_hour(self, hour: u8) -> Result<Self, error::ComponentRange> {
+        Ok(const_try!(self.local_datetime.replace_hour(hour)).assume_offset(self.offset))
+    }
+
+    /// Replace the minutes within the hour.
+    ///
+    /// ```rust
+    /// # use time::macros::datetime;
+    /// assert_eq!(
+    ///     datetime!(2022 - 02 - 18 01:02:03.004_005_006 +01).replace_minute(7),
+    ///     Ok(datetime!(2022 - 02 - 18 01:07:03.004_005_006 +01))
+    /// );
+    /// assert!(datetime!(2022 - 02 - 18 01:02:03.004_005_006 +01).replace_minute(60).is_err()); // 60 isn't a valid minute
+    /// ```
+    pub const fn replace_minute(self, minute: u8) -> Result<Self, error::ComponentRange> {
+        Ok(const_try!(self.local_datetime.replace_minute(minute)).assume_offset(self.offset))
+    }
+
+    /// Replace the seconds within the minute.
+    ///
+    /// ```rust
+    /// # use time::macros::datetime;
+    /// assert_eq!(
+    ///     datetime!(2022 - 02 - 18 01:02:03.004_005_006 +01).replace_second(7),
+    ///     Ok(datetime!(2022 - 02 - 18 01:02:07.004_005_006 +01))
+    /// );
+    /// assert!(datetime!(2022 - 02 - 18 01:02:03.004_005_006 +01).replace_second(60).is_err()); // 60 isn't a valid second
+    /// ```
+    pub const fn replace_second(self, second: u8) -> Result<Self, error::ComponentRange> {
+        Ok(const_try!(self.local_datetime.replace_second(second)).assume_offset(self.offset))
+    }
+
+    /// Replace the milliseconds within the second.
+    ///
+    /// ```rust
+    /// # use time::macros::datetime;
+    /// assert_eq!(
+    ///     datetime!(2022 - 02 - 18 01:02:03.004_005_006 +01).replace_millisecond(7),
+    ///     Ok(datetime!(2022 - 02 - 18 01:02:03.007 +01))
+    /// );
+    /// assert!(datetime!(2022 - 02 - 18 01:02:03.004_005_006 +01).replace_millisecond(1_000).is_err()); // 1_000 isn't a valid millisecond
+    /// ```
+    pub const fn replace_millisecond(
+        self,
+        millisecond: u16,
+    ) -> Result<Self, error::ComponentRange> {
+        Ok(
+            const_try!(self.local_datetime.replace_millisecond(millisecond))
+                .assume_offset(self.offset),
+        )
+    }
+
+    /// Replace the microseconds within the second.
+    ///
+    /// ```rust
+    /// # use time::macros::datetime;
+    /// assert_eq!(
+    ///     datetime!(2022 - 02 - 18 01:02:03.004_005_006 +01).replace_microsecond(7_008),
+    ///     Ok(datetime!(2022 - 02 - 18 01:02:03.007_008 +01))
+    /// );
+    /// assert!(datetime!(2022 - 02 - 18 01:02:03.004_005_006 +01).replace_microsecond(1_000_000).is_err()); // 1_000_000 isn't a valid microsecond
+    /// ```
+    pub const fn replace_microsecond(
+        self,
+        microsecond: u32,
+    ) -> Result<Self, error::ComponentRange> {
+        Ok(
+            const_try!(self.local_datetime.replace_microsecond(microsecond))
+                .assume_offset(self.offset),
+        )
+    }
+
+    /// Replace the nanoseconds within the second.
+    ///
+    /// ```rust
+    /// # use time::macros::datetime;
+    /// assert_eq!(
+    ///     datetime!(2022 - 02 - 18 01:02:03.004_005_006 +01).replace_nanosecond(7_008_009),
+    ///     Ok(datetime!(2022 - 02 - 18 01:02:03.007_008_009 +01))
+    /// );
+    /// assert!(datetime!(2022 - 02 - 18 01:02:03.004_005_006 +01).replace_nanosecond(1_000_000_000).is_err()); // 1_000_000_000 isn't a valid nanosecond
+    /// ```
+    pub const fn replace_nanosecond(self, nanosecond: u32) -> Result<Self, error::ComponentRange> {
+        Ok(
+            const_try!(self.local_datetime.replace_nanosecond(nanosecond))
+                .assume_offset(self.offset),
+        )
     }
 }
 // endregion replacement
 
 // region: formatting & parsing
 #[cfg(feature = "formatting")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "formatting")))]
 impl OffsetDateTime {
     /// Format the `OffsetDateTime` using the provided [format
     /// description](crate::format_description).
     pub fn format_into(
         self,
         output: &mut impl io::Write,
-        format: &impl Formattable,
+        format: &(impl Formattable + ?Sized),
     ) -> Result<usize, error::Format> {
-        let local = self.utc_datetime.utc_to_offset(self.offset);
         format.format_into(
             output,
-            Some(local.date),
-            Some(local.time),
+            Some(self.date()),
+            Some(self.time()),
             Some(self.offset),
         )
     }
@@ -805,14 +1076,12 @@ impl OffsetDateTime {
     /// );
     /// # Ok::<_, time::Error>(())
     /// ```
-    pub fn format(self, format: &impl Formattable) -> Result<String, error::Format> {
-        let local = self.utc_datetime.utc_to_offset(self.offset);
-        format.format(Some(local.date), Some(local.time), Some(self.offset))
+    pub fn format(self, format: &(impl Formattable + ?Sized)) -> Result<String, error::Format> {
+        format.format(Some(self.date()), Some(self.time()), Some(self.offset))
     }
 }
 
 #[cfg(feature = "parsing")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "parsing")))]
 impl OffsetDateTime {
     /// Parse an `OffsetDateTime` from the input using the provided [format
     /// description](crate::format_description).
@@ -829,9 +1098,35 @@ impl OffsetDateTime {
     /// );
     /// # Ok::<_, time::Error>(())
     /// ```
-    pub fn parse(input: &str, description: &impl Parsable) -> Result<Self, error::Parse> {
+    pub fn parse(
+        input: &str,
+        description: &(impl Parsable + ?Sized),
+    ) -> Result<Self, error::Parse> {
         description.parse_offset_date_time(input.as_bytes())
     }
+
+    /// A helper method to check if the `OffsetDateTime` is a valid representation of a leap second.
+    /// Leap seconds, when parsed, are represented as the preceding nanosecond. However, leap
+    /// seconds can only occur as the last second of a month UTC.
+    pub(crate) const fn is_valid_leap_second_stand_in(self) -> bool {
+        // This comparison doesn't need to be adjusted for the stored offset, so check it first for
+        // speed.
+        if self.nanosecond() != 999_999_999 {
+            return false;
+        }
+
+        let (year, ordinal, time) = self.to_offset_raw(UtcOffset::UTC);
+
+        let date = match Date::from_ordinal_date(year, ordinal) {
+            Ok(date) => date,
+            Err(_) => return false,
+        };
+
+        time.hour() == 23
+            && time.minute() == 59
+            && time.second() == 59
+            && date.day() == util::days_in_year_month(year, date.month())
+    }
 }
 
 impl fmt::Display for OffsetDateTime {
@@ -839,12 +1134,18 @@ impl fmt::Display for OffsetDateTime {
         write!(f, "{} {} {}", self.date(), self.time(), self.offset)
     }
 }
+
+impl fmt::Debug for OffsetDateTime {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt::Display::fmt(self, f)
+    }
+}
 // endregion formatting & parsing
 
 // region: trait impls
 impl PartialEq for OffsetDateTime {
     fn eq(&self, rhs: &Self) -> bool {
-        self.utc_datetime.eq(&rhs.utc_datetime)
+        self.to_offset_raw(UtcOffset::UTC) == rhs.to_offset_raw(UtcOffset::UTC)
     }
 }
 
@@ -856,7 +1157,8 @@ impl PartialOrd for OffsetDateTime {
 
 impl Ord for OffsetDateTime {
     fn cmp(&self, rhs: &Self) -> Ordering {
-        self.utc_datetime.cmp(&rhs.utc_datetime)
+        self.to_offset_raw(UtcOffset::UTC)
+            .cmp(&rhs.to_offset_raw(UtcOffset::UTC))
     }
 }
 
@@ -864,7 +1166,7 @@ impl Hash for OffsetDateTime {
     fn hash<H: Hasher>(&self, hasher: &mut H) {
         // We need to distinguish this from a `PrimitiveDateTime`, which would otherwise conflict.
         hasher.write(b"OffsetDateTime");
-        self.utc_datetime.hash(hasher);
+        self.to_offset_raw(UtcOffset::UTC).hash(hasher);
     }
 }
 
@@ -875,9 +1177,7 @@ where
     type Output = Self;
 
     fn add(self, rhs: T) -> Self::Output {
-        (self.utc_datetime + rhs)
-            .assume_utc()
-            .to_offset(self.offset)
+        (self.local_datetime + rhs).assume_offset(self.offset)
     }
 }
 
@@ -890,9 +1190,7 @@ where
     type Output = Self;
 
     fn sub(self, rhs: T) -> Self::Output {
-        (self.utc_datetime - rhs)
-            .assume_utc()
-            .to_offset(self.offset)
+        (self.local_datetime - rhs).assume_offset(self.offset)
     }
 }
 
@@ -902,12 +1200,13 @@ impl Sub for OffsetDateTime {
     type Output = Duration;
 
     fn sub(self, rhs: Self) -> Self::Output {
-        self.utc_datetime - rhs.utc_datetime
+        let adjustment =
+            Duration::seconds((self.offset.whole_seconds() - rhs.offset.whole_seconds()) as i64);
+        self.local_datetime - rhs.local_datetime - adjustment
     }
 }
 
 #[cfg(feature = "std")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
 impl Add<Duration> for SystemTime {
     type Output = Self;
 
@@ -915,22 +1214,17 @@ impl Add<Duration> for SystemTime {
         if duration.is_zero() {
             self
         } else if duration.is_positive() {
-            self + duration.abs_std()
+            self + duration.unsigned_abs()
         } else {
             debug_assert!(duration.is_negative());
-            self - duration.abs_std()
+            self - duration.unsigned_abs()
         }
     }
 }
 
-impl_add_assign!(SystemTime:
-    #[cfg(feature = "std")]
-    #[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
-    Duration
-);
+impl_add_assign!(SystemTime: #[cfg(feature = "std")] Duration);
 
 #[cfg(feature = "std")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
 impl Sub<Duration> for SystemTime {
     type Output = Self;
 
@@ -939,14 +1233,9 @@ impl Sub<Duration> for SystemTime {
     }
 }
 
-impl_sub_assign!(SystemTime:
-    #[cfg(feature = "std")]
-    #[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
-    Duration
-);
+impl_sub_assign!(SystemTime: #[cfg(feature = "std")] Duration);
 
 #[cfg(feature = "std")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
 impl Sub<SystemTime> for OffsetDateTime {
     type Output = Duration;
 
@@ -956,7 +1245,6 @@ impl Sub<SystemTime> for OffsetDateTime {
 }
 
 #[cfg(feature = "std")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
 impl Sub<OffsetDateTime> for SystemTime {
     type Output = Duration;
 
@@ -966,7 +1254,6 @@ impl Sub<OffsetDateTime> for SystemTime {
 }
 
 #[cfg(feature = "std")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
 impl PartialEq<SystemTime> for OffsetDateTime {
     fn eq(&self, rhs: &SystemTime) -> bool {
         self == &Self::from(*rhs)
@@ -974,7 +1261,6 @@ impl PartialEq<SystemTime> for OffsetDateTime {
 }
 
 #[cfg(feature = "std")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
 impl PartialEq<OffsetDateTime> for SystemTime {
     fn eq(&self, rhs: &OffsetDateTime) -> bool {
         &OffsetDateTime::from(*self) == rhs
@@ -982,7 +1268,6 @@ impl PartialEq<OffsetDateTime> for SystemTime {
 }
 
 #[cfg(feature = "std")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
 impl PartialOrd<SystemTime> for OffsetDateTime {
     fn partial_cmp(&self, other: &SystemTime) -> Option<Ordering> {
         self.partial_cmp(&Self::from(*other))
@@ -990,7 +1275,6 @@ impl PartialOrd<SystemTime> for OffsetDateTime {
 }
 
 #[cfg(feature = "std")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
 impl PartialOrd<OffsetDateTime> for SystemTime {
     fn partial_cmp(&self, other: &OffsetDateTime) -> Option<Ordering> {
         OffsetDateTime::from(*self).partial_cmp(other)
@@ -998,7 +1282,6 @@ impl PartialOrd<OffsetDateTime> for SystemTime {
 }
 
 #[cfg(feature = "std")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
 impl From<SystemTime> for OffsetDateTime {
     fn from(system_time: SystemTime) -> Self {
         match system_time.duration_since(SystemTime::UNIX_EPOCH) {
@@ -1010,7 +1293,6 @@ impl From<SystemTime> for OffsetDateTime {
 
 #[allow(clippy::fallible_impl_from)] // caused by `debug_assert!`
 #[cfg(feature = "std")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "std")))]
 impl From<OffsetDateTime> for SystemTime {
     fn from(datetime: OffsetDateTime) -> Self {
         let duration = datetime - OffsetDateTime::UNIX_EPOCH;
@@ -1018,11 +1300,40 @@ impl From<OffsetDateTime> for SystemTime {
         if duration.is_zero() {
             Self::UNIX_EPOCH
         } else if duration.is_positive() {
-            Self::UNIX_EPOCH + duration.abs_std()
+            Self::UNIX_EPOCH + duration.unsigned_abs()
         } else {
             debug_assert!(duration.is_negative());
-            Self::UNIX_EPOCH - duration.abs_std()
+            Self::UNIX_EPOCH - duration.unsigned_abs()
         }
     }
 }
+
+#[allow(clippy::fallible_impl_from)]
+#[cfg(all(
+    target_arch = "wasm32",
+    not(any(target_os = "emscripten", target_os = "wasi")),
+    feature = "wasm-bindgen"
+))]
+impl From<js_sys::Date> for OffsetDateTime {
+    fn from(js_date: js_sys::Date) -> Self {
+        // get_time() returns milliseconds
+        let timestamp_nanos = (js_date.get_time() * 1_000_000.0) as i128;
+        Self::from_unix_timestamp_nanos(timestamp_nanos)
+            .expect("invalid timestamp: Timestamp cannot fit in range")
+    }
+}
+
+#[cfg(all(
+    target_arch = "wasm32",
+    not(any(target_os = "emscripten", target_os = "wasi")),
+    feature = "wasm-bindgen"
+))]
+impl From<OffsetDateTime> for js_sys::Date {
+    fn from(datetime: OffsetDateTime) -> Self {
+        // new Date() takes milliseconds
+        let timestamp = (datetime.unix_timestamp_nanos() / 1_000_000) as f64;
+        js_sys::Date::new(&timestamp.into())
+    }
+}
+
 // endregion trait impls
