@@ -1,3 +1,5 @@
+//! The [`Time`] struct and its associated `impl`s.
+
 use core::fmt;
 use core::ops::{Add, Sub};
 use core::time::Duration as StdDuration;
@@ -15,7 +17,7 @@ use crate::{error, Duration};
 /// perform niche value optimization.
 #[repr(u8)]
 #[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
-enum Padding {
+pub(crate) enum Padding {
     #[allow(clippy::missing_docs_in_private_items)]
     Optimize,
 }
@@ -40,17 +42,6 @@ pub struct Time {
     padding: Padding,
 }
 
-impl fmt::Debug for Time {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("Time")
-            .field("hour", &self.hour)
-            .field("minute", &self.minute)
-            .field("second", &self.second)
-            .field("nanosecond", &self.nanosecond)
-            .finish()
-    }
-}
-
 impl Time {
     /// Create a `Time` that is exactly midnight.
     ///
@@ -60,6 +51,16 @@ impl Time {
     /// ```
     pub const MIDNIGHT: Self = Self::__from_hms_nanos_unchecked(0, 0, 0, 0);
 
+    /// The smallest value that can be represented by `Time`.
+    ///
+    /// `00:00:00.0`
+    pub(crate) const MIN: Self = Self::__from_hms_nanos_unchecked(0, 0, 0, 0);
+
+    /// The largest value that can be represented by `Time`.
+    ///
+    /// `23:59:59.999_999_999`
+    pub(crate) const MAX: Self = Self::__from_hms_nanos_unchecked(23, 59, 59, 999_999_999);
+
     // region: constructors
     /// Create a `Time` from its components.
     #[doc(hidden)]
@@ -69,6 +70,11 @@ impl Time {
         second: u8,
         nanosecond: u32,
     ) -> Self {
+        debug_assert!(hour < 24);
+        debug_assert!(minute < 60);
+        debug_assert!(second < 60);
+        debug_assert!(nanosecond < 1_000_000_000);
+
         Self {
             hour,
             minute,
@@ -328,8 +334,8 @@ impl Time {
     // endregion getters
 
     // region: arithmetic helpers
-    /// Add the sub-day time of the [`Duration`] to the `Time`. Wraps on overflow, returning the
-    /// necessary whether the date is the following day.
+    /// Add the sub-day time of the [`Duration`] to the `Time`. Wraps on overflow, returning whether
+    /// the date is different.
     pub(crate) const fn adjusting_add(self, duration: Duration) -> (DateAdjustment, Self) {
         let mut nanoseconds = self.nanosecond as i32 + duration.subsec_nanoseconds();
         let mut seconds = self.second as i8 + (duration.whole_seconds() % 60) as i8;
@@ -359,6 +365,37 @@ impl Time {
         )
     }
 
+    /// Subtract the sub-day time of the [`Duration`] to the `Time`. Wraps on overflow, returning
+    /// whether the date is different.
+    pub(crate) const fn adjusting_sub(self, duration: Duration) -> (DateAdjustment, Self) {
+        let mut nanoseconds = self.nanosecond as i32 - duration.subsec_nanoseconds();
+        let mut seconds = self.second as i8 - (duration.whole_seconds() % 60) as i8;
+        let mut minutes = self.minute as i8 - (duration.whole_minutes() % 60) as i8;
+        let mut hours = self.hour as i8 - (duration.whole_hours() % 24) as i8;
+        let mut date_adjustment = DateAdjustment::None;
+
+        cascade!(nanoseconds in 0..1_000_000_000 => seconds);
+        cascade!(seconds in 0..60 => minutes);
+        cascade!(minutes in 0..60 => hours);
+        if hours >= 24 {
+            hours -= 24;
+            date_adjustment = DateAdjustment::Next;
+        } else if hours < 0 {
+            hours += 24;
+            date_adjustment = DateAdjustment::Previous;
+        }
+
+        (
+            date_adjustment,
+            Self::__from_hms_nanos_unchecked(
+                hours as _,
+                minutes as _,
+                seconds as _,
+                nanoseconds as _,
+            ),
+        )
+    }
+
     /// Add the sub-day time of the [`std::time::Duration`] to the `Time`. Wraps on overflow,
     /// returning whether the date is the previous date as the first element of the tuple.
     pub(crate) const fn adjusting_add_std(self, duration: StdDuration) -> (bool, Self) {
@@ -405,17 +442,150 @@ impl Time {
         )
     }
     // endregion arithmetic helpers
+
+    // region: replacement
+    /// Replace the clock hour.
+    ///
+    /// ```rust
+    /// # use time::macros::time;
+    /// assert_eq!(
+    ///     time!(01:02:03.004_005_006).replace_hour(7),
+    ///     Ok(time!(07:02:03.004_005_006))
+    /// );
+    /// assert!(time!(01:02:03.004_005_006).replace_hour(24).is_err()); // 24 isn't a valid hour
+    /// ```
+    #[must_use = "This method does not mutate the original `Time`."]
+    pub const fn replace_hour(self, hour: u8) -> Result<Self, error::ComponentRange> {
+        ensure_value_in_range!(hour in 0 => 23);
+        Ok(Self::__from_hms_nanos_unchecked(
+            hour,
+            self.minute,
+            self.second,
+            self.nanosecond,
+        ))
+    }
+
+    /// Replace the minutes within the hour.
+    ///
+    /// ```rust
+    /// # use time::macros::time;
+    /// assert_eq!(
+    ///     time!(01:02:03.004_005_006).replace_minute(7),
+    ///     Ok(time!(01:07:03.004_005_006))
+    /// );
+    /// assert!(time!(01:02:03.004_005_006).replace_minute(60).is_err()); // 60 isn't a valid minute
+    /// ```
+    #[must_use = "This method does not mutate the original `Time`."]
+    pub const fn replace_minute(self, minute: u8) -> Result<Self, error::ComponentRange> {
+        ensure_value_in_range!(minute in 0 => 59);
+        Ok(Self::__from_hms_nanos_unchecked(
+            self.hour,
+            minute,
+            self.second,
+            self.nanosecond,
+        ))
+    }
+
+    /// Replace the seconds within the minute.
+    ///
+    /// ```rust
+    /// # use time::macros::time;
+    /// assert_eq!(
+    ///     time!(01:02:03.004_005_006).replace_second(7),
+    ///     Ok(time!(01:02:07.004_005_006))
+    /// );
+    /// assert!(time!(01:02:03.004_005_006).replace_second(60).is_err()); // 60 isn't a valid second
+    /// ```
+    #[must_use = "This method does not mutate the original `Time`."]
+    pub const fn replace_second(self, second: u8) -> Result<Self, error::ComponentRange> {
+        ensure_value_in_range!(second in 0 => 59);
+        Ok(Self::__from_hms_nanos_unchecked(
+            self.hour,
+            self.minute,
+            second,
+            self.nanosecond,
+        ))
+    }
+
+    /// Replace the milliseconds within the second.
+    ///
+    /// ```rust
+    /// # use time::macros::time;
+    /// assert_eq!(
+    ///     time!(01:02:03.004_005_006).replace_millisecond(7),
+    ///     Ok(time!(01:02:03.007))
+    /// );
+    /// assert!(time!(01:02:03.004_005_006).replace_millisecond(1_000).is_err()); // 1_000 isn't a valid millisecond
+    /// ```
+    #[must_use = "This method does not mutate the original `Time`."]
+    pub const fn replace_millisecond(
+        self,
+        millisecond: u16,
+    ) -> Result<Self, error::ComponentRange> {
+        ensure_value_in_range!(millisecond in 0 => 999);
+        Ok(Self::__from_hms_nanos_unchecked(
+            self.hour,
+            self.minute,
+            self.second,
+            millisecond as u32 * 1_000_000,
+        ))
+    }
+
+    /// Replace the microseconds within the second.
+    ///
+    /// ```rust
+    /// # use time::macros::time;
+    /// assert_eq!(
+    ///     time!(01:02:03.004_005_006).replace_microsecond(7_008),
+    ///     Ok(time!(01:02:03.007_008))
+    /// );
+    /// assert!(time!(01:02:03.004_005_006).replace_microsecond(1_000_000).is_err()); // 1_000_000 isn't a valid microsecond
+    /// ```
+    #[must_use = "This method does not mutate the original `Time`."]
+    pub const fn replace_microsecond(
+        self,
+        microsecond: u32,
+    ) -> Result<Self, error::ComponentRange> {
+        ensure_value_in_range!(microsecond in 0 => 999_999);
+        Ok(Self::__from_hms_nanos_unchecked(
+            self.hour,
+            self.minute,
+            self.second,
+            microsecond * 1000,
+        ))
+    }
+
+    /// Replace the nanoseconds within the second.
+    ///
+    /// ```rust
+    /// # use time::macros::time;
+    /// assert_eq!(
+    ///     time!(01:02:03.004_005_006).replace_nanosecond(7_008_009),
+    ///     Ok(time!(01:02:03.007_008_009))
+    /// );
+    /// assert!(time!(01:02:03.004_005_006).replace_nanosecond(1_000_000_000).is_err()); // 1_000_000_000 isn't a valid nanosecond
+    /// ```
+    #[must_use = "This method does not mutate the original `Time`."]
+    pub const fn replace_nanosecond(self, nanosecond: u32) -> Result<Self, error::ComponentRange> {
+        ensure_value_in_range!(nanosecond in 0 => 999_999_999);
+        Ok(Self::__from_hms_nanos_unchecked(
+            self.hour,
+            self.minute,
+            self.second,
+            nanosecond,
+        ))
+    }
+    // endregion replacement
 }
 
 // region: formatting & parsing
 #[cfg(feature = "formatting")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "formatting")))]
 impl Time {
     /// Format the `Time` using the provided [format description](crate::format_description).
     pub fn format_into(
         self,
         output: &mut impl io::Write,
-        format: &impl Formattable,
+        format: &(impl Formattable + ?Sized),
     ) -> Result<usize, crate::error::Format> {
         format.format_into(output, None, Some(self), None)
     }
@@ -428,13 +598,15 @@ impl Time {
     /// assert_eq!(time!(12:00).format(&format)?, "12:00:00");
     /// # Ok::<_, time::Error>(())
     /// ```
-    pub fn format(self, format: &impl Formattable) -> Result<String, crate::error::Format> {
+    pub fn format(
+        self,
+        format: &(impl Formattable + ?Sized),
+    ) -> Result<String, crate::error::Format> {
         format.format(None, Some(self), None)
     }
 }
 
 #[cfg(feature = "parsing")]
-#[cfg_attr(__time_03_docs, doc(cfg(feature = "parsing")))]
 impl Time {
     /// Parse a `Time` from the input using the provided [format
     /// description](crate::format_description).
@@ -445,7 +617,10 @@ impl Time {
     /// assert_eq!(Time::parse("12:00:00", &format)?, time!(12:00));
     /// # Ok::<_, time::Error>(())
     /// ```
-    pub fn parse(input: &str, description: &impl Parsable) -> Result<Self, error::Parse> {
+    pub fn parse(
+        input: &str,
+        description: &(impl Parsable + ?Sized),
+    ) -> Result<Self, error::Parse> {
         description.parse_time(input.as_bytes())
     }
 }
@@ -465,15 +640,17 @@ impl fmt::Display for Time {
         };
         write!(
             f,
-            "{}:{:02}:{:02}.{:0width$}",
-            self.hour,
-            self.minute,
-            self.second,
-            value,
-            width = width
+            "{}:{:02}:{:02}.{value:0width$}",
+            self.hour, self.minute, self.second,
         )
     }
 }
+
+impl fmt::Debug for Time {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt::Display::fmt(self, f)
+    }
+}
 // endregion formatting & parsing
 
 // region: trait impls
@@ -520,7 +697,7 @@ impl Sub<Duration> for Time {
     /// assert_eq!(time!(23:59:59) - (-2).seconds(), time!(0:00:01));
     /// ```
     fn sub(self, duration: Duration) -> Self::Output {
-        self + -duration
+        self.adjusting_sub(duration).1
     }
 }
 
@@ -557,15 +734,20 @@ impl Sub for Time {
     fn sub(self, rhs: Self) -> Self::Output {
         let hour_diff = (self.hour as i8) - (rhs.hour as i8);
         let minute_diff = (self.minute as i8) - (rhs.minute as i8);
-        let mut second_diff = (self.second as i8) - (rhs.second as i8);
-        let mut nanosecond_diff = (self.nanosecond as i32) - (rhs.nanosecond as i32);
+        let second_diff = (self.second as i8) - (rhs.second as i8);
+        let nanosecond_diff = (self.nanosecond as i32) - (rhs.nanosecond as i32);
 
-        cascade!(nanosecond_diff in 0..1_000_000_000 => second_diff);
+        let seconds = hour_diff as i64 * 3_600 + minute_diff as i64 * 60 + second_diff as i64;
 
-        Duration::new_unchecked(
-            hour_diff as i64 * 3_600 + minute_diff as i64 * 60 + second_diff as i64,
-            nanosecond_diff,
-        )
+        let (seconds, nanoseconds) = if seconds > 0 && nanosecond_diff < 0 {
+            (seconds - 1, nanosecond_diff + 1_000_000_000)
+        } else if seconds < 0 && nanosecond_diff > 0 {
+            (seconds + 1, nanosecond_diff - 1_000_000_000)
+        } else {
+            (seconds, nanosecond_diff)
+        };
+
+        Duration::new_unchecked(seconds, nanoseconds)
     }
 }
 // endregion trait impls
